==> bfinter-0.3.tar.bz2: bfinter-0.3/bfinter.c <==
// dividuums brainfuck interpreter 0.3
//
// ---------------------------------
//
// Copyright (c) 2000 Florian Wesch <fw@dividuum.de>. All Rights Reserved.     
//                                                                              
// Permission to use, copy, modify, and distribute this software                
// and its documentation for any purposes and without               
// fee is hereby granted provided that this copyright notice                    
// appears in all copies.                                                       
//                                                                              
// Of course, this soft is provided "as is" without express or implied          
// warranty of any kind.        
//
// -----------------------------
// Version 0.2
// - ist jetzt binary-save
// - eingabe funktioniert
//
// Version 0.2
// - sollte jetzt binary-save sein
//
// Version 0.1
// - enthlt wahrscheinlich noch ein paar bugs
// - u.U. nicht standar{d|t}-konform (siehe FIXME)
//   ? Welchen Datentyp haben die Speicherzellen
//   ? Was passiert bei overflows (momentan wird mit OVERFLOW/UNDERFLOW abgebrochen
//
// TODO
// - Cache fr Sprnge

#include "bfinter.h"

int executebf(unsigned char *programm,  int progsize, 
              unsigned char *input,     int inputsize, 
              unsigned char *output,    int *outputsize, 
              int maxoutputsize,        int memsize,
              int maxinstructions) {
    
    char *memory;
      
    int  ip=0; // instructionpointer
    int  mp=0; // memorypointer
    
    int  status=0; // interpreterstatus (s.o.)

    int  depth; // fr die [ Suche
    int  suchip;   

    int read=0; // gelesene bytes der eingabe

    int runtime=0;
    
    *outputsize=0; // geschriebene bytes

    memory=(unsigned char*)calloc(memsize,1); 

    while (ip<progsize) {
        //printf("ip=%d programm[ip]=%c mp=%d memory[mp]=%d\n",ip,programm[ip],mp,memory[mp]);
        switch (programm[ip]) {
            case BF_OPINCV: // Wert an aktueller Speicherstelle erhhen
                //if (memory[mp]==255) { // FIXME Was ist mit Overflows?
                //    status=BF_OVERFLOW;
                //    goto raus;
                //}
                memory[mp]++;
                break;
            case BF_OPDECV: // Wert an aktueller Speicherstelle erniedrigen
                //if (memory[mp]==0) { // FIXME Was ist mit Underflows?
                //    status=BF_UNDERFLOW;
                //    goto raus;
                //}
                memory[mp]--;
                break;
            case BF_OPINCP: // Speicherzellenpointer erniedrigen
                mp++;
                if (mp>memsize) // FIXME Ok so? 
                    mp=memsize;
                break;
            case BF_OPDECP:
                mp--;
                if (mp<0) // FIXME Ok so?
                    mp=0;
                break;
            case BF_OPLOOPS: // Zu zugehrigem ] springen
                if (memory[mp]==0) { 
                    //printf("sprung zu --> ]\n");
                    depth=1;
                    suchip=ip+1;

                    while (suchip<progsize && depth>0) { 
                        //printf("suchip=%d depth=%d programm[suchip]=%c\n",suchip,depth,programm[suchip]);
                        switch (programm[suchip]) {
                            case BF_OPLOOPE: depth--; break;
                            case BF_OPLOOPS: depth++; break;
                        }
                        suchip++;
                    }

                    if (depth==0) { // suchip befindet sich hinter dem gefundenen ]
                        ip=suchip-1;
                    } else {
                        status=BF_UNMATCHEDOPEN;
                        goto raus;
                    }
                }
                break;
            case BF_OPLOOPE: // Zu zugehrigem [ springen
                //printf("sprung zu [ <---\n");
                depth=1;
                suchip=ip-1;
                while ((suchip>=0) && (depth>0)) { // FIXME besser machen 
                    //printf("suchip=%d depth=%d programm[suchip]=%c\n",suchip,depth,programm[suchip]);
                    switch (programm[suchip]) {
                        case BF_OPLOOPE: depth++; break;
                        case BF_OPLOOPS: depth--; break;
                    }
                    suchip--;
                }
                                      
                if (depth==0) {
                    ip=suchip;
                } else {
                    status=BF_UNMATCHEDCLOSE;
                    goto raus;
                }
                break;
            case BF_OPIN:
#ifndef INTERACTIVE
                if (read<inputsize) {
                    memory[mp]=input[read++];
                } else {
                    status=BF_INPUTEMPTY;
                    goto raus;
                }
#else
#error "Not yet supported"
#endif
                break;
            case BF_OPOUT:
                //printf("AUSGABE memory[%d]=%c (%d)\n",mp,memory[mp],memory[mp]);
                if (*outputsize<maxoutputsize) {
                    output[*outputsize]=memory[mp];
                    (*outputsize)++;
                } else {
                    status=BF_OUTPUTFULL;
                    goto raus;
                }
                break;
            default:
                // Unbekannte Opcodes berspringen
                //status=UNKNOWNOPCODE;
                //goto raus;
                break;
        }
        
        // Laufzeitbegrenzung
        if (maxinstructions!=BF_UNLIMITED && runtime>maxinstructions) {
            status=BF_TOOMANYINST;
            goto raus;
        }
        
        ip++;
        runtime++; 
    }
    //printf("SUCCESS output %d\n",written);
    status=BF_SUCCESS;
raus:
    //printf("%d\n",runtime);
    free(memory);
    return status;
}

==> bfinter-0.5.tar.bz2: bfinter-0.5/bfinter.c <==
// dividuums brainfuck interpreter 0.5
//
// ---------------------------------
//
// Copyright (c) 2001 Florian Wesch <fw@dividuum.de>. All Rights Reserved.     
//                                                                              
// Permission to use, copy, modify, and distribute this software                
// and its documentation for any purposes and without               
// fee is hereby granted provided that this copyright notice                    
// appears in all copies.                                                       
//                                                                              
// Of course, this soft is provided "as is" without express or implied          
// warranty of any kind.        
//
// -----------------------------
//
// Version 0.5 (1.3.2001)
// - cacheerstellung gefixt 
// 
// Version 0.4 (1.3.2001)
// - jumpcache fr extra schnelles ausfhren
// 
// Version 0.3 (1.3.2001)
// - ist jetzt binary-save
// - eingabe funktioniert
//
// Version 0.2 (1.3.2001)
// - sollte jetzt binary-save sein
//
// Version 0.1 (28.2.2001)
// - enthlt wahrscheinlich noch ein paar bugs
// - u.U. nicht standar{d|t}-konform (siehe FIXME)
//   ? Welchen Datentyp haben die Speicherzellen
//   ? Was passiert bei overflows (momentan wird mit OVERFLOW/UNDERFLOW abgebrochen
//

#include "bfinter.h"

int executebf(unsigned char *programm,  int progsize, 
              unsigned char *input,     int inputsize, 
              unsigned char *output,    int *outputsize, 
              int maxoutputsize,        int memsize,
              int maxinstructions) {
    
    unsigned char *memory;
    
    int *jumpcache; 
    int *jumpstack;
      
    int  ip=0; // instructionpointer
    int  mp=0; // memorypointer
    
    int  status=0; // interpreterstatus (siehe bfinter.h)

    int runtime=0; // anzahl ausgefhrter instruktionen
    
    int depth=-1; // fr die jumpcache erstellung
    int i;  // dito
    
    int read=0; // gelesene bytes der eingabe
    *outputsize=0; // geschriebene bytes der ausgabe
    
    memory   =(unsigned char*)calloc(memsize,1);  // Datenspeicher
   
    jumpstack=(int*)malloc(BF_MAXNESTEDLOOPS*sizeof(int)); 
    jumpcache=(int*)malloc(progsize*sizeof(int)); 

    // Jumpcache aufbauen
    for (i=0; i<progsize; i++) {
        switch (programm[i]) {
            case BF_OPLOOPS:
                depth++;
                if (depth>=BF_MAXNESTEDLOOPS) {
                    status=BF_TOOMANYNESTEDLOOPS;
                    goto raus;
                }
                jumpstack[depth]=i;
                break;
            case BF_OPLOOPE:
                if (depth<0) {
                    status=BF_UNMATCHEDCLOSE;
                    goto raus;
                }
                jumpcache[i]=jumpstack[depth];
                jumpcache[jumpstack[depth]]=i;
                depth--;
                break;
            default:
                jumpcache[i]=i;
        }
    }
    if (depth>=0) {
        status=BF_UNMATCHEDOPEN;
        goto raus;
    }

    //for (i=0; i<progsize; i++) printf("@%d %c -> %d\n",i,programm[i],jumpcache[i]);

    while (ip<progsize) {
        // printf("ip=%d programm[ip]=%c mp=%d memory[mp]=%d\n",ip,programm[ip],mp,memory[mp]);
        switch (programm[ip]) {
            case BF_OPINCV: // Wert an aktueller Speicherstelle erhhen
                //if (memory[mp]==255) { // FIXME Was ist mit Overflows?
                //    status=BF_OVERFLOW;
                //    goto raus;
                //}
                memory[mp]++;
                break;
            case BF_OPDECV: // Wert an aktueller Speicherstelle erniedrigen
                //if (memory[mp]==0) { // FIXME Was ist mit Underflows?
                //    status=BF_UNDERFLOW;
                //    goto raus;
                //}
                memory[mp]--;
                break;
            case BF_OPINCP: // Speicherzellenpointer erniedrigen
                mp++;
                if (mp>memsize) // FIXME Ok so? 
                    mp=memsize;
                break;
            case BF_OPDECP:
                mp--;
                if (mp<0) // FIXME Ok so?
                    mp=0;
                break;
            case BF_OPLOOPS: // Zu zugehrigem ] springen
                if (memory[mp]==0) { 
                    ip=jumpcache[ip];
                }
                break;
            case BF_OPLOOPE: // Zu zugehrigem [ springen
                ip=jumpcache[ip]-1;
                break;
            case BF_OPIN:
#ifndef INTERACTIVE
                if (read<inputsize) {
                    memory[mp]=input[read++];
                } else {
                    status=BF_INPUTEMPTY;
                    goto raus;
                }
#else
#error "Not yet supported"
#endif
                break;
            case BF_OPOUT:
                //printf("AUSGABE memory[%d]=%c (%d)\n",mp,memory[mp],memory[mp]);
                if (*outputsize<maxoutputsize) {
                    output[*outputsize]=memory[mp];
                    (*outputsize)++;
                } else {
                    status=BF_OUTPUTFULL;
                    goto raus;
                }
                break;
            default:
                // Unbekannte Opcodes berspringen
                //status=UNKNOWNOPCODE;
                //goto raus;
                break;
        }
        
        // Laufzeitbegrenzung
        if (maxinstructions!=BF_UNLIMITED && runtime>maxinstructions) {
            status=BF_TOOMANYINST;
            goto raus;
        }
        
        ip++;
        runtime++; 
    }
    //printf("SUCCESS output %d\n",written);
    status=BF_SUCCESS;
raus:
    //printf("%d\n",runtime);
    free(jumpcache);
    free(jumpstack);
    free(memory);
    return status;
}
